@@ -295,7 +295,8 @@ class _MapperConfig(object):
                             # produces nested proxies, so we are only
                             # looking one level deep right now.
                             if isinstance(ret, InspectionAttr) and \
-                                    ret._is_internal_proxy:
+                                    ret._is_internal_proxy and not isinstance(
+                                        ret.original_property, MapperProperty):
                                 ret = ret.descriptor
 
                             dict_[name] = column_copies[obj] = ret
@@ -533,11 +534,6 @@ class _MapperConfig(object):
                     )
                 # add any columns declared here to the inherited table.
                 for c in declared_columns:
-                    if c.primary_key:
-                        raise exc.ArgumentError(
-                            "Can't place primary key columns on an inherited "
-                            "class with no table."
-                        )
                     if c.name in inherited_table.c:
                         if inherited_table.c[c.name] is c:
                             continue
@@ -546,6 +542,11 @@ class _MapperConfig(object):
                             "existing column '%s'" %
                             (c, cls, inherited_table.c[c.name])
                         )
+                    if c.primary_key:
+                        raise exc.ArgumentError(
+                            "Can't place primary key columns on an inherited "
+                            "class with no table."
+                        )
                     inherited_table.append_column(c)
                     if inherited_mapped_table is not None and \
                             inherited_mapped_table is not inherited_table:
