@@ -4,14 +4,17 @@ from sqlalchemy.testing import eq_, is_
 from sqlalchemy import Column, Table, DDL, MetaData, TIMESTAMP, \
     DefaultClause, String, Integer, Text, UnicodeText, SmallInteger,\
     NCHAR, LargeBinary, DateTime, select, UniqueConstraint, Unicode,\
-    BigInteger
+    BigInteger, Index, ForeignKey
 from sqlalchemy import event
 from sqlalchemy import sql
+from sqlalchemy import exc
 from sqlalchemy import inspect
 from sqlalchemy.dialects.mysql import base as mysql
 from sqlalchemy.dialects.mysql import reflection as _reflection
 from sqlalchemy.testing import fixtures, AssertsExecutionResults
 from sqlalchemy import testing
+from sqlalchemy.testing import assert_raises_message, expect_warnings
+import re
 
 
 class TypeReflectionTest(fixtures.TestBase):
@@ -184,7 +187,7 @@ class TypeReflectionTest(fixtures.TestBase):
     def test_legacy_enum_types(self):
 
         specs = [
-            (mysql.ENUM("''","'fleem'"), mysql.ENUM("''","'fleem'")),  # noqa
+            (mysql.ENUM("''", "'fleem'"), mysql.ENUM("''", "'fleem'")),
         ]
 
         self._run_test(specs, ['enums'])
@@ -231,9 +234,9 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
         assert reflected.c.c5.default is None
         assert reflected.c.c5.server_default is None
         assert reflected.c.c6.default is None
-        eq_(
-            str(reflected.c.c6.server_default.arg).upper(),
-            "CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP"
+        assert re.match(
+            r"CURRENT_TIMESTAMP(\(\))? ON UPDATE CURRENT_TIMESTAMP(\(\))?",
+            str(reflected.c.c6.server_default.arg).upper()
         )
         reflected.create()
         try:
@@ -249,9 +252,9 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
         assert reflected.c.c5.default is None
         assert reflected.c.c5.server_default is None
         assert reflected.c.c6.default is None
-        eq_(
-            str(reflected.c.c6.server_default.arg).upper(),
-            "CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP"
+        assert re.match(
+            r"CURRENT_TIMESTAMP(\(\))? ON UPDATE CURRENT_TIMESTAMP(\(\))?",
+            str(reflected.c.c6.server_default.arg).upper()
         )
 
     def test_reflection_with_table_options(self):
@@ -261,7 +264,7 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
             'mysql_def', MetaData(testing.db),
             Column('c1', Integer()),
             mysql_engine='MEMORY',
-            mysql_comment=comment,
+            comment=comment,
             mysql_default_charset='utf8',
             mysql_auto_increment='5',
             mysql_avg_row_length='3',
@@ -278,7 +281,7 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
             def_table.drop()
 
         assert def_table.kwargs['mysql_engine'] == 'MEMORY'
-        assert def_table.kwargs['mysql_comment'] == comment
+        assert def_table.comment == comment
         assert def_table.kwargs['mysql_default_charset'] == 'utf8'
         assert def_table.kwargs['mysql_auto_increment'] == '5'
         assert def_table.kwargs['mysql_avg_row_length'] == '3'
@@ -286,6 +289,8 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
         assert def_table.kwargs['mysql_connection'] == 'fish'
 
         assert reflected.kwargs['mysql_engine'] == 'MEMORY'
+
+        assert reflected.comment == comment
         assert reflected.kwargs['mysql_comment'] == comment
         assert reflected.kwargs['mysql_default charset'] == 'utf8'
         assert reflected.kwargs['mysql_avg_row_length'] == '3'
@@ -399,7 +404,10 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
 
     @testing.provide_metadata
     def test_view_reflection(self):
-        Table('x', self.metadata, Column('a', Integer), Column('b', String(50)))
+        Table('x',
+              self.metadata,
+              Column('a', Integer),
+              Column('b', String(50)))
         self.metadata.create_all()
 
         with testing.db.connect() as conn:
@@ -427,6 +435,34 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
                 [('a', mysql.INTEGER), ('b', mysql.VARCHAR)]
             )
 
+    @testing.provide_metadata
+    def test_skip_not_describable(self):
+        @event.listens_for(self.metadata, "before_drop")
+        def cleanup(*arg, **kw):
+            with testing.db.connect() as conn:
+                conn.execute("DROP TABLE IF EXISTS test_t1")
+                conn.execute("DROP TABLE IF EXISTS test_t2")
+                conn.execute("DROP VIEW IF EXISTS test_v")
+
+        with testing.db.connect() as conn:
+            conn.execute("CREATE TABLE test_t1 (id INTEGER)")
+            conn.execute("CREATE TABLE test_t2 (id INTEGER)")
+            conn.execute("CREATE VIEW test_v AS SELECT id FROM test_t1" )
+            conn.execute("DROP TABLE test_t1")
+
+            m = MetaData()
+            with expect_warnings(
+                "Skipping .* Table or view named .?test_v.? could not be "
+                "reflected: .* references invalid table"
+            ):
+                m.reflect(views=True, bind=conn)
+            eq_(m.tables['test_t2'].name, "test_t2")
+
+            assert_raises_message(
+                exc.UnreflectableTableError,
+                "references invalid table",
+                Table, 'test_v', MetaData(), autoload_with=conn
+            )
 
     @testing.exclude('mysql', '<', (5, 0, 0), 'no information_schema support')
     def test_system_views(self):
@@ -446,6 +482,11 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
         # this is ideally one table, but older MySQL versions choke
         # on the multiple TIMESTAMP columns
 
+        row = testing.db.execute(
+            "show variables like '%%explicit_defaults_for_timestamp%%'"
+        ).first()
+        explicit_defaults_for_timestamp = row[1].lower() in ('on', '1', 'true')
+
         reflected = []
         for idx, cols in enumerate([
             [
@@ -476,6 +517,11 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
                 for d in inspect(testing.db).get_columns("nn_t%d" % idx)
             )
 
+        if testing.db.dialect._is_mariadb_102:
+            current_timestamp = "current_timestamp()"
+        else:
+            current_timestamp = "CURRENT_TIMESTAMP"
+
         eq_(
             reflected,
             [
@@ -484,15 +530,23 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
                 {'name': 'z', 'nullable': True, 'default': None},
                 {'name': 'q', 'nullable': True, 'default': None},
                 {'name': 'p', 'nullable': True,
-                 'default': 'CURRENT_TIMESTAMP'},
+                 'default': current_timestamp},
                 {'name': 'r', 'nullable': False,
-                 'default': "CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP"},
+                 'default': None if explicit_defaults_for_timestamp else
+                 "%(current_timestamp)s ON UPDATE %(current_timestamp)s" %
+                 {"current_timestamp": current_timestamp}},
                 {'name': 's', 'nullable': False,
-                 'default': 'CURRENT_TIMESTAMP'},
-                {'name': 't', 'nullable': False,
-                 'default': "CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP"},
-                {'name': 'u', 'nullable': False,
-                 'default': 'CURRENT_TIMESTAMP'},
+                 'default': current_timestamp},
+                {'name': 't',
+                 'nullable': True if explicit_defaults_for_timestamp else
+                 False,
+                 'default': None if explicit_defaults_for_timestamp else
+                 "%(current_timestamp)s ON UPDATE %(current_timestamp)s" %
+                 {"current_timestamp": current_timestamp}},
+                {'name': 'u',
+                 'nullable': True if explicit_defaults_for_timestamp else
+                 False,
+                 'default': current_timestamp},
             ]
         )
 
@@ -528,8 +582,68 @@ class ReflectionTest(fixtures.TestBase, AssertsExecutionResults):
         self.assert_(indexes['uc_a'].unique)
         self.assert_('uc_a' not in constraints)
 
+    @testing.provide_metadata
+    def test_non_column_index(self):
+        m1 = self.metadata
+        t1 = Table(
+            'add_ix', m1, Column('x', String(50)), mysql_engine='InnoDB')
+        Index('foo_idx', t1.c.x.desc())
+        m1.create_all()
+
+        insp = inspect(testing.db)
+        eq_(
+            insp.get_indexes("add_ix"),
+            [{'name': 'foo_idx', 'column_names': ['x'], 'unique': False}]
+        )
+
+    @testing.provide_metadata
+    def test_case_sensitive_column_constraint_reflection(self):
+        # test for issue #4344 which works around
+        # MySQL 8.0 bug https://bugs.mysql.com/bug.php?id=88718
+
+        m1 = self.metadata
+
+        Table(
+            'Track', m1, Column('TrackID', Integer, primary_key=True)
+        )
+        Table(
+            'Track', m1, Column('TrackID', Integer, primary_key=True),
+            schema=testing.config.test_schema
+        )
+        Table(
+            'PlaylistTrack', m1, Column('id', Integer, primary_key=True),
+            Column('TrackID',
+                   ForeignKey('Track.TrackID', name='FK_PlaylistTrackId')),
+            Column(
+                'TTrackID',
+                ForeignKey(
+                    '%s.Track.TrackID' % (testing.config.test_schema,),
+                    name='FK_PlaylistTTrackId'
+                )
+            )
+        )
+        m1.create_all()
+
+        eq_(
+            inspect(testing.db).get_foreign_keys('PlaylistTrack'),
+            [
+                {'name': 'FK_PlaylistTTrackId',
+                 'constrained_columns': ['TTrackID'],
+                 'referred_schema': testing.config.test_schema,
+                 'referred_table': 'Track',
+                 'referred_columns': ['TrackID'], 'options': {}},
+                {'name': 'FK_PlaylistTrackId',
+                 'constrained_columns': ['TrackID'],
+                 'referred_schema': None,
+                 'referred_table': 'Track',
+                 'referred_columns': ['TrackID'], 'options': {}}
+            ]
+        )
+
 
 class RawReflectionTest(fixtures.TestBase):
+    __backend__ = True
+
     def setup(self):
         dialect = mysql.dialect()
         self.parser = _reflection.MySQLTableDefinitionParser(
@@ -552,23 +666,81 @@ class RawReflectionTest(fixtures.TestBase):
             '  PRIMARY KEY (`id`) USING BTREE KEY_BLOCK_SIZE  = 16')
         assert not regex.match(
             '  PRIMARY KEY (`id`) USING BTREE KEY_BLOCK_SIZE = = 16')
+        assert regex.match(
+            "  KEY (`id`) USING BTREE COMMENT 'comment'")
+        # `SHOW CREATE TABLE` returns COMMENT '''comment'
+        # after creating table with COMMENT '\'comment'
+        assert regex.match(
+            "  KEY (`id`) USING BTREE COMMENT '''comment'")
+        assert regex.match(
+            "  KEY (`id`) USING BTREE COMMENT 'comment'''")
+        assert regex.match(
+            "  KEY (`id`) USING BTREE COMMENT 'prefix''suffix'")
+        assert regex.match(
+            "  KEY (`id`) USING BTREE COMMENT 'prefix''text''suffix'")
+
+    def test_key_reflection_columns(self):
+        regex = self.parser._re_key
+        exprs = self.parser._re_keyexprs
+        m = regex.match(
+            "  KEY (`id`) USING BTREE COMMENT '''comment'")
+        eq_(m.group("columns"), '`id`')
+
+        m = regex.match(
+            "  KEY (`x`, `y`) USING BTREE")
+        eq_(m.group("columns"), '`x`, `y`')
+
+        eq_(
+            exprs.findall(m.group("columns")),
+            [("x", "", ""), ("y", "", "")]
+        )
+
+        m = regex.match(
+            "  KEY (`x`(25), `y`(15)) USING BTREE")
+        eq_(m.group("columns"), '`x`(25), `y`(15)')
+        eq_(
+            exprs.findall(m.group("columns")),
+            [("x", "25", ""), ("y", "15", "")]
+        )
+
+        m = regex.match(
+            "  KEY (`x`(25) DESC, `y`(15) ASC) USING BTREE")
+        eq_(m.group("columns"), '`x`(25) DESC, `y`(15) ASC')
+        eq_(
+            exprs.findall(m.group("columns")),
+            [("x", "25", "DESC"), ("y", "15", "ASC")]
+        )
+
+        m = regex.match(
+            "  KEY `foo_idx` (`x` DESC)")
+        eq_(m.group("columns"), '`x` DESC')
+        eq_(
+            exprs.findall(m.group("columns")),
+            [("x", "", "DESC")]
+        )
+
+        eq_(
+            exprs.findall(m.group("columns")),
+            [("x", "", "DESC")]
+        )
+
+        m = regex.match(
+            "  KEY `foo_idx` (`x` DESC, `y` ASC)")
+        eq_(m.group("columns"), '`x` DESC, `y` ASC')
 
     def test_fk_reflection(self):
-        regex = self.parser._re_constraint
+        regex = self.parser._re_fk_constraint
 
         m = regex.match('  CONSTRAINT `addresses_user_id_fkey` '
                         'FOREIGN KEY (`user_id`) '
                         'REFERENCES `users` (`id`) '
                         'ON DELETE CASCADE ON UPDATE CASCADE')
         eq_(m.groups(), ('addresses_user_id_fkey', '`user_id`',
-                            '`users`', '`id`', None, 'CASCADE', 'CASCADE'))
-
+                         '`users`', '`id`', None, 'CASCADE', 'CASCADE'))
 
         m = regex.match('  CONSTRAINT `addresses_user_id_fkey` '
                         'FOREIGN KEY (`user_id`) '
                         'REFERENCES `users` (`id`) '
                         'ON DELETE CASCADE ON UPDATE SET NULL')
         eq_(m.groups(), ('addresses_user_id_fkey', '`user_id`',
-                            '`users`', '`id`', None, 'CASCADE', 'SET NULL'))
-
-
+                         '`users`', '`id`', None, 'CASCADE', 'SET NULL'))
