@@ -13,7 +13,8 @@ from sqlalchemy import Table, Column, MetaData, Integer, String, \
 from sqlalchemy import exc
 import sqlalchemy as sa
 from sqlalchemy.dialects.postgresql import base as postgresql
-from sqlalchemy.dialects.postgresql import ARRAY
+from sqlalchemy.dialects.postgresql import ARRAY, INTERVAL, TSRANGE
+from sqlalchemy.dialects.postgresql import ExcludeConstraint
 import re
 
 
@@ -706,8 +707,8 @@ class ReflectionTest(fixtures.TestBase):
             tmp.sort()
             r1, r2 = [idx[1] for idx in tmp]
             assert r1.name == 'idx2'
-            assert r1.unique == True
-            assert r2.unique == False
+            assert r1.unique is True
+            assert r2.unique is False
             assert [t2.c.id] == r1.columns
             assert [t2.c.name] == r2.columns
 
@@ -907,6 +908,41 @@ class ReflectionTest(fixtures.TestBase):
                 'schema': 'public'
             }])
 
+    @testing.provide_metadata
+    def test_inspect_enums_case_sensitive(self):
+        enum_type = postgresql.ENUM(
+            'CapsOne', 'CapsTwo', name='UpperCase', metadata=self.metadata)
+        enum_type.create(testing.db)
+        inspector = reflection.Inspector.from_engine(testing.db)
+        eq_(inspector.get_enums(), [
+            {
+                'visible': True,
+                'labels': ['CapsOne', 'CapsTwo'],
+                'name': 'UpperCase',
+                'schema': 'public'
+            }])
+
+    @testing.provide_metadata
+    def test_inspect_enums_case_sensitive_from_table(self):
+        enum_type = postgresql.ENUM(
+            'CapsOne', 'CapsTwo', name='UpperCase', metadata=self.metadata)
+
+        t = Table('t', self.metadata, Column('q', enum_type))
+
+        enum_type.create(testing.db)
+        t.create(testing.db)
+
+        inspector = reflection.Inspector.from_engine(testing.db)
+        cols = inspector.get_columns("t")
+        cols[0]['type'] = (cols[0]['type'].name, cols[0]['type'].enums)
+        eq_(cols, [
+            {
+                'name': 'q',
+                'type': ('UpperCase', ['CapsOne', 'CapsTwo']),
+                'nullable': True, 'default': None,
+                'autoincrement': False, 'comment': None}
+        ])
+
     @testing.provide_metadata
     def test_inspect_enums_star(self):
         enum_type = postgresql.ENUM(
@@ -971,6 +1007,36 @@ class ReflectionTest(fixtures.TestBase):
         self.assert_('uc_a' not in indexes)
         self.assert_('uc_a' in constraints)
 
+    @testing.requires.btree_gist
+    @testing.provide_metadata
+    def test_reflection_with_exclude_constraint(self):
+        m = self.metadata
+        Table(
+            't', m,
+            Column('id', Integer, primary_key=True),
+            Column('period', TSRANGE),
+            ExcludeConstraint(('period', '&&'), name='quarters_period_excl')
+        )
+
+        m.create_all()
+
+        insp = inspect(testing.db)
+
+        # PostgreSQL will create an implicit index for an exclude constraint.
+        # we don't reflect the EXCLUDE yet.
+        eq_(
+            insp.get_indexes('t'),
+            [{'unique': False, 'name': 'quarters_period_excl',
+              'duplicates_constraint': 'quarters_period_excl',
+              'dialect_options': {'postgresql_using': 'gist'},
+              'column_names': ['period']}]
+        )
+
+        # reflection corrects for the dupe
+        reflected = Table('t', MetaData(testing.db), autoload=True)
+
+        eq_(set(reflected.indexes), set())
+
     @testing.provide_metadata
     def test_reflect_unique_index(self):
         insp = inspect(testing.db)
@@ -1026,6 +1092,7 @@ class ReflectionTest(fixtures.TestBase):
         })
 
 
+
 class CustomTypeReflectionTest(fixtures.TestBase):
 
     class CustomType(object):
@@ -1054,7 +1121,7 @@ class CustomTypeReflectionTest(fixtures.TestBase):
         ]:
             column_info = dialect._get_column_info(
                 'colname', sch, None, False,
-                {}, {}, 'public')
+                {}, {}, 'public', None)
             assert isinstance(column_info['type'], self.CustomType)
             eq_(column_info['type'].arg1, args[0])
             eq_(column_info['type'].arg2, args[1])
@@ -1069,3 +1136,61 @@ class CustomTypeReflectionTest(fixtures.TestBase):
         dialect.ischema_names = dialect.ischema_names.copy()
         dialect.ischema_names['my_custom_type'] = self.CustomType
         self._assert_reflected(dialect)
+
+
+class IntervalReflectionTest(fixtures.TestBase):
+    __only_on__ = 'postgresql'
+    __backend__ = True
+
+    def test_interval_types(self):
+        for sym in [
+            "YEAR",
+            "MONTH",
+            "DAY",
+            "HOUR",
+            "MINUTE",
+            "SECOND",
+            "YEAR TO MONTH",
+            "DAY TO HOUR",
+            "DAY TO MINUTE",
+            "DAY TO SECOND",
+            "HOUR TO MINUTE",
+            "HOUR TO SECOND",
+            "MINUTE TO SECOND",
+        ]:
+            self._test_interval_symbol(sym)
+
+    @testing.provide_metadata
+    def _test_interval_symbol(self, sym):
+        t = Table(
+            'i_test', self.metadata,
+            Column('id', Integer, primary_key=True),
+            Column('data1', INTERVAL(fields=sym)),
+        )
+        t.create(testing.db)
+
+        columns = {
+            rec['name']: rec
+            for rec in inspect(testing.db).get_columns("i_test")
+        }
+        assert isinstance(columns["data1"]["type"], INTERVAL)
+        eq_(columns["data1"]["type"].fields, sym.lower())
+        eq_(columns["data1"]["type"].precision, None)
+
+    @testing.provide_metadata
+    def test_interval_precision(self):
+        t = Table(
+            'i_test', self.metadata,
+            Column('id', Integer, primary_key=True),
+            Column('data1', INTERVAL(precision=6)),
+        )
+        t.create(testing.db)
+
+        columns = {
+            rec['name']: rec
+            for rec in inspect(testing.db).get_columns("i_test")
+        }
+        assert isinstance(columns["data1"]["type"], INTERVAL)
+        eq_(columns["data1"]["type"].fields, None)
+        eq_(columns["data1"]["type"].precision, 6)
+
