@@ -1,8 +1,10 @@
 from sqlalchemy.testing import fixtures, eq_, is_, is_not_
 from sqlalchemy import testing
 from sqlalchemy.testing import assert_raises_message
+from sqlalchemy.testing import expect_warnings
 from sqlalchemy.sql import column, desc, asc, literal, collate, null, \
     true, false, any_, all_
+from sqlalchemy.sql import sqltypes
 from sqlalchemy.sql.expression import BinaryExpression, \
     ClauseList, Grouping, \
     UnaryExpression, select, union, func, tuple_
@@ -11,11 +13,11 @@ import operator
 from sqlalchemy import String, Integer, LargeBinary
 from sqlalchemy import exc
 from sqlalchemy.engine import default
-from sqlalchemy.sql.elements import _literal_as_text
+from sqlalchemy.sql.elements import _literal_as_text, Label
 from sqlalchemy.schema import Column, Table, MetaData
 from sqlalchemy.sql import compiler
 from sqlalchemy.types import TypeEngine, TypeDecorator, UserDefinedType, \
-    Boolean, NullType, MatchType, Indexable, Concatenable, ARRAY, JSON, \
+    Boolean, MatchType, Indexable, Concatenable, ARRAY, JSON, \
     DateTime
 from sqlalchemy.dialects import mysql, firebird, postgresql, oracle, \
     sqlite, mssql
@@ -62,10 +64,16 @@ class DefaultColumnComparatorTest(fixtures.TestBase):
 
         self._loop_test(operator, right)
 
+        if operators.is_comparison(operator):
+            is_(
+                left.comparator.operate(operator, right).type,
+                sqltypes.BOOLEANTYPE
+            )
+
     def _loop_test(self, operator, *arg):
-        l = LoopOperate()
+        loop = LoopOperate()
         is_(
-            operator(l, *arg),
+            operator(loop, *arg),
             operator
         )
 
@@ -184,7 +192,7 @@ class DefaultColumnComparatorTest(fixtures.TestBase):
         foo = ClauseList()
         assert_raises_message(
             exc.InvalidRequestError,
-            r"in_\(\) accepts either a list of expressions or a selectable:",
+            r"in_\(\) accepts either a list of expressions, a selectable",
             left.in_, [foo]
         )
 
@@ -193,7 +201,7 @@ class DefaultColumnComparatorTest(fixtures.TestBase):
         right = column('right')
         assert_raises_message(
             exc.InvalidRequestError,
-            r"in_\(\) accepts either a list of expressions or a selectable:",
+            r"in_\(\) accepts either a list of expressions, a selectable",
             left.in_, right
         )
 
@@ -210,7 +218,7 @@ class DefaultColumnComparatorTest(fixtures.TestBase):
         right = column('right', HasGetitem)
         assert_raises_message(
             exc.InvalidRequestError,
-            r"in_\(\) accepts either a list of expressions or a selectable:",
+            r"in_\(\) accepts either a list of expressions, a selectable",
             left.in_, right
         )
 
@@ -628,7 +636,7 @@ class ExtensionOperatorTest(fixtures.TestBase, testing.AssertsCompiledSQL):
                     return self.op("->")(index)
 
         col = Column('x', MyType())
-        assert not isinstance(col, collections.Iterable)
+        assert not isinstance(col, util.collections_abc.Iterable)
 
     def test_lshift(self):
         class MyType(UserDefinedType):
@@ -764,7 +772,7 @@ class JSONIndexOpTest(fixtures.TestBase, testing.AssertsCompiledSQL):
         col = Column('x', self.MyType())
 
         self.assert_compile(
-            col[8] != None,
+            col[8] != None,  # noqa
             "(x -> :x_1) IS NOT NULL"
         )
 
@@ -774,7 +782,7 @@ class JSONIndexOpTest(fixtures.TestBase, testing.AssertsCompiledSQL):
         col2 = Column('y', Integer())
 
         self.assert_compile(
-            col[col2 + 8] != None,
+            col[col2 + 8] != None,  # noqa
             "(x -> (y + :y_1)) IS NOT NULL",
             checkparams={'y_1': 8}
         )
@@ -1306,7 +1314,7 @@ class OperatorPrecedenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
 
     def test_operator_precedence_1(self):
         self.assert_compile(
-            self.table2.select((self.table2.c.field == 5) == None),
+            self.table2.select((self.table2.c.field == 5) == None),  # noqa
             "SELECT op.field FROM op WHERE (op.field = :field_1) IS NULL")
 
     def test_operator_precedence_2(self):
@@ -1389,19 +1397,19 @@ class OperatorPrecedenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
     def test_operator_precedence_collate_1(self):
         self.assert_compile(
             self.table1.c.name == literal('foo').collate('utf-8'),
-            "mytable.name = (:param_1 COLLATE utf-8)"
+            'mytable.name = (:param_1 COLLATE "utf-8")'
         )
 
     def test_operator_precedence_collate_2(self):
         self.assert_compile(
             (self.table1.c.name == literal('foo')).collate('utf-8'),
-            "mytable.name = :param_1 COLLATE utf-8"
+            'mytable.name = :param_1 COLLATE "utf-8"'
         )
 
     def test_operator_precedence_collate_3(self):
         self.assert_compile(
             self.table1.c.name.collate('utf-8') == 'foo',
-            "(mytable.name COLLATE utf-8) = :param_1"
+            '(mytable.name COLLATE "utf-8") = :param_1'
         )
 
     def test_operator_precedence_collate_4(self):
@@ -1410,8 +1418,8 @@ class OperatorPrecedenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
                 (self.table1.c.name == literal('foo')).collate('utf-8'),
                 (self.table2.c.field == literal('bar')).collate('utf-8'),
             ),
-            "mytable.name = :param_1 COLLATE utf-8 "
-            "AND op.field = :param_2 COLLATE utf-8"
+            'mytable.name = :param_1 COLLATE "utf-8" '
+            'AND op.field = :param_2 COLLATE "utf-8"'
         )
 
     def test_operator_precedence_collate_5(self):
@@ -1419,7 +1427,7 @@ class OperatorPrecedenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             select([self.table1.c.name]).order_by(
                 self.table1.c.name.collate('utf-8').desc()),
             "SELECT mytable.name FROM mytable "
-            "ORDER BY mytable.name COLLATE utf-8 DESC"
+            'ORDER BY mytable.name COLLATE "utf-8" DESC'
         )
 
     def test_operator_precedence_collate_6(self):
@@ -1427,7 +1435,7 @@ class OperatorPrecedenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             select([self.table1.c.name]).order_by(
                 self.table1.c.name.collate('utf-8').desc().nullslast()),
             "SELECT mytable.name FROM mytable "
-            "ORDER BY mytable.name COLLATE utf-8 DESC NULLS LAST"
+            'ORDER BY mytable.name COLLATE "utf-8" DESC NULLS LAST'
         )
 
     def test_operator_precedence_collate_7(self):
@@ -1435,7 +1443,7 @@ class OperatorPrecedenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             select([self.table1.c.name]).order_by(
                 self.table1.c.name.collate('utf-8').asc()),
             "SELECT mytable.name FROM mytable "
-            "ORDER BY mytable.name COLLATE utf-8 ASC"
+            'ORDER BY mytable.name COLLATE "utf-8" ASC'
         )
 
     def test_commutative_operators(self):
@@ -1471,6 +1479,13 @@ class OperatorPrecedenceTest(fixtures.TestBase, testing.AssertsCompiledSQL):
         self.assert_compile(op2, "mytable.myid hoho :myid_1 lala :param_1")
         self.assert_compile(op3, "(mytable.myid hoho :myid_1) lala :param_1")
 
+    def test_is_eq_precedence_flat(self):
+        self.assert_compile(
+            (self.table1.c.name == null()) !=
+            (self.table1.c.description == null()),
+            "(mytable.name IS NULL) != (mytable.description IS NULL)",
+        )
+
 
 class OperatorAssociativityTest(fixtures.TestBase, testing.AssertsCompiledSQL):
     __dialect__ = 'default'
@@ -1565,11 +1580,11 @@ class OperatorAssociativityTest(fixtures.TestBase, testing.AssertsCompiledSQL):
 
     def test_associativity_22(self):
         f = column('f')
-        self.assert_compile((f==f) == f, '(f = f) = f')
+        self.assert_compile((f == f) == f, '(f = f) = f')
 
     def test_associativity_23(self):
         f = column('f')
-        self.assert_compile((f!=f) != f, '(f != f) != f')
+        self.assert_compile((f != f) != f, '(f != f) != f')
 
 
 class IsDistinctFromTest(fixtures.TestBase, testing.AssertsCompiledSQL):
@@ -1638,6 +1653,11 @@ class InTest(fixtures.TestBase, testing.AssertsCompiledSQL):
         column('othername', String)
     )
 
+    def _dialect(self, empty_in_strategy="static"):
+        return default.DefaultDialect(
+            empty_in_strategy=empty_in_strategy
+        )
+
     def test_in_1(self):
         self.assert_compile(self.table1.c.myid.in_(['a']),
                             "mytable.myid IN (:myid_1)")
@@ -1738,8 +1758,9 @@ class InTest(fixtures.TestBase, testing.AssertsCompiledSQL):
 
     def test_in_21(self):
         self.assert_compile(~self.table1.c.myid.in_(
-                            select([self.table2.c.otherid])),
-                            "mytable.myid NOT IN (SELECT myothertable.otherid FROM myothertable)")
+            select([self.table2.c.otherid])),
+            "mytable.myid NOT IN "
+            "(SELECT myothertable.otherid FROM myothertable)")
 
     def test_in_22(self):
         self.assert_compile(
@@ -1750,11 +1771,6 @@ class InTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             "FROM myothertable)"
         )
 
-    @testing.emits_warning('.*empty sequence.*')
-    def test_in_23(self):
-        self.assert_compile(self.table1.c.myid.in_([]),
-                            "mytable.myid != mytable.myid")
-
     def test_in_24(self):
         self.assert_compile(
             select([self.table1.c.myid.in_(select([self.table2.c.otherid]))]),
@@ -1778,7 +1794,8 @@ class InTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             )
         ), "mytable.myid IN ("
             "SELECT mytable.myid FROM mytable WHERE mytable.myid = :myid_1 "
-            "UNION SELECT mytable.myid FROM mytable WHERE mytable.myid = :myid_2)")
+            "UNION SELECT mytable.myid FROM mytable "
+            "WHERE mytable.myid = :myid_2)")
 
     def test_in_27(self):
         # test that putting a select in an IN clause does not
@@ -1790,8 +1807,9 @@ class InTest(fixtures.TestBase, testing.AssertsCompiledSQL):
                        order_by=[self.table2.c.othername],
                        limit=10, correlate=False)
             ),
-                from_obj=[self.table1.join(self.table2,
-                                           self.table1.c.myid == self.table2.c.otherid)],
+                from_obj=[self.table1.join(
+                    self.table2,
+                    self.table1.c.myid == self.table2.c.otherid)],
                 order_by=[self.table1.c.myid]
             ),
             "SELECT mytable.myid, "
@@ -1809,15 +1827,53 @@ class InTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             "mytable.myid IN (NULL)"
         )
 
-    @testing.emits_warning('.*empty sequence.*')
-    def test_in_29(self):
-        self.assert_compile(self.table1.c.myid.notin_([]),
-                            "mytable.myid = mytable.myid")
+    def test_empty_in_dynamic_1(self):
+        self.assert_compile(self.table1.c.myid.in_([]),
+                            "mytable.myid != mytable.myid",
+                            dialect=self._dialect("dynamic"))
 
-    @testing.emits_warning('.*empty sequence.*')
-    def test_in_30(self):
+    def test_empty_in_dynamic_2(self):
+        self.assert_compile(self.table1.c.myid.notin_([]),
+                            "mytable.myid = mytable.myid",
+                            dialect=self._dialect("dynamic"))
+
+    def test_empty_in_dynamic_3(self):
         self.assert_compile(~self.table1.c.myid.in_([]),
-                            "mytable.myid = mytable.myid")
+                            "mytable.myid = mytable.myid",
+                            dialect=self._dialect("dynamic"))
+
+    def test_empty_in_dynamic_warn_1(self):
+        with testing.expect_warnings(
+                "The IN-predicate was invoked with an empty sequence."):
+            self.assert_compile(self.table1.c.myid.in_([]),
+                                "mytable.myid != mytable.myid",
+                                dialect=self._dialect("dynamic_warn"))
+
+    def test_empty_in_dynamic_warn_2(self):
+        with testing.expect_warnings(
+                "The IN-predicate was invoked with an empty sequence."):
+            self.assert_compile(self.table1.c.myid.notin_([]),
+                                "mytable.myid = mytable.myid",
+                                dialect=self._dialect("dynamic_warn"))
+
+    def test_empty_in_dynamic_warn_3(self):
+        with testing.expect_warnings(
+                "The IN-predicate was invoked with an empty sequence."):
+            self.assert_compile(~self.table1.c.myid.in_([]),
+                                "mytable.myid = mytable.myid",
+                                dialect=self._dialect("dynamic_warn"))
+
+    def test_empty_in_static_1(self):
+        self.assert_compile(self.table1.c.myid.in_([]),
+                            "1 != 1")
+
+    def test_empty_in_static_2(self):
+        self.assert_compile(self.table1.c.myid.notin_([]),
+                            "1 = 1")
+
+    def test_empty_in_static_3(self):
+        self.assert_compile(~self.table1.c.myid.in_([]),
+                            "1 = 1")
 
 
 class MathOperatorTest(fixtures.TestBase, testing.AssertsCompiledSQL):
@@ -2038,6 +2094,52 @@ class NegationTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             self.table1.c.myid.type,
         )
 
+    def test_negate_operator_label(self):
+        orig_expr = or_(
+            self.table1.c.myid == 1, self.table1.c.myid == 2).label('foo')
+        expr = not_(orig_expr)
+        isinstance(expr, Label)
+        eq_(expr.name, 'foo')
+        is_not_(expr, orig_expr)
+        is_(expr._element.operator, operator.inv)  # e.g. and not false_
+
+        self.assert_compile(
+            expr,
+            "NOT (mytable.myid = :myid_1 OR mytable.myid = :myid_2)",
+            dialect=default.DefaultDialect(supports_native_boolean=False)
+        )
+
+    def test_negate_operator_self_group(self):
+        orig_expr = or_(
+            self.table1.c.myid == 1, self.table1.c.myid == 2).self_group()
+        expr = not_(orig_expr)
+        is_not_(expr, orig_expr)
+
+        self.assert_compile(
+            expr,
+            "NOT (mytable.myid = :myid_1 OR mytable.myid = :myid_2)",
+            dialect=default.DefaultDialect(supports_native_boolean=False)
+        )
+
+    def test_implicitly_boolean(self):
+        # test for expressions that the database always considers as boolean
+        # even if there is no boolean datatype.
+        assert not self.table1.c.myid._is_implicitly_boolean
+        assert (self.table1.c.myid == 5)._is_implicitly_boolean
+        assert (self.table1.c.myid == 5).self_group()._is_implicitly_boolean
+        assert (self.table1.c.myid == 5).label('x')._is_implicitly_boolean
+        assert not_(self.table1.c.myid == 5)._is_implicitly_boolean
+        assert or_(
+            self.table1.c.myid == 5, self.table1.c.myid == 7
+        )._is_implicitly_boolean
+        assert not column('x', Boolean)._is_implicitly_boolean
+        assert not (self.table1.c.myid + 5)._is_implicitly_boolean
+        assert not not_(column('x', Boolean))._is_implicitly_boolean
+        assert not select([self.table1.c.myid]).\
+            as_scalar()._is_implicitly_boolean
+        assert not text("x = y")._is_implicitly_boolean
+        assert not literal_column("x = y")._is_implicitly_boolean
+
 
 class LikeTest(fixtures.TestBase, testing.AssertsCompiledSQL):
     __dialect__ = 'default'
@@ -2217,43 +2319,57 @@ class ComposedLikeOperatorsTest(fixtures.TestBase, testing.AssertsCompiledSQL):
     def test_contains(self):
         self.assert_compile(
             column('x').contains('y'),
-            "x LIKE '%%' || :x_1 || '%%'",
+            "x LIKE '%' || :x_1 || '%'",
             checkparams={'x_1': 'y'}
         )
 
     def test_contains_escape(self):
         self.assert_compile(
-            column('x').contains('y', escape='\\'),
-            "x LIKE '%%' || :x_1 || '%%' ESCAPE '\\'",
-            checkparams={'x_1': 'y'}
+            column('x').contains('a%b_c', escape='\\'),
+            "x LIKE '%' || :x_1 || '%' ESCAPE '\\'",
+            checkparams={'x_1': 'a%b_c'}
+        )
+
+    def test_contains_autoescape(self):
+        self.assert_compile(
+            column('x').contains('a%b_c/d', autoescape=True),
+            "x LIKE '%' || :x_1 || '%' ESCAPE '/'",
+            checkparams={'x_1': 'a/%b/_c//d'}
         )
 
     def test_contains_literal(self):
         self.assert_compile(
             column('x').contains(literal_column('y')),
-            "x LIKE '%%' || y || '%%'",
+            "x LIKE '%' || y || '%'",
             checkparams={}
         )
 
     def test_contains_text(self):
         self.assert_compile(
             column('x').contains(text('y')),
-            "x LIKE '%%' || y || '%%'",
+            "x LIKE '%' || y || '%'",
             checkparams={}
         )
 
     def test_not_contains(self):
         self.assert_compile(
             ~column('x').contains('y'),
-            "x NOT LIKE '%%' || :x_1 || '%%'",
+            "x NOT LIKE '%' || :x_1 || '%'",
             checkparams={'x_1': 'y'}
         )
 
     def test_not_contains_escape(self):
         self.assert_compile(
-            ~column('x').contains('y', escape='\\'),
-            "x NOT LIKE '%%' || :x_1 || '%%' ESCAPE '\\'",
-            checkparams={'x_1': 'y'}
+            ~column('x').contains('a%b_c', escape='\\'),
+            "x NOT LIKE '%' || :x_1 || '%' ESCAPE '\\'",
+            checkparams={'x_1': 'a%b_c'}
+        )
+
+    def test_not_contains_autoescape(self):
+        self.assert_compile(
+            ~column('x').contains('a%b_c/d', autoescape=True),
+            "x NOT LIKE '%' || :x_1 || '%' ESCAPE '/'",
+            checkparams={'x_1': 'a/%b/_c//d'}
         )
 
     def test_contains_concat(self):
@@ -2288,45 +2404,122 @@ class ComposedLikeOperatorsTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             dialect=mysql.dialect()
         )
 
+    def test_like(self):
+        self.assert_compile(
+            column('x').like('y'),
+            "x LIKE :x_1",
+            checkparams={'x_1': 'y'}
+        )
+
+    def test_like_escape(self):
+        self.assert_compile(
+            column('x').like('a%b_c', escape='\\'),
+            "x LIKE :x_1 ESCAPE '\\'",
+            checkparams={'x_1': 'a%b_c'}
+        )
+
+    def test_ilike(self):
+        self.assert_compile(
+            column('x').ilike('y'),
+            "lower(x) LIKE lower(:x_1)",
+            checkparams={'x_1': 'y'}
+        )
+
+    def test_ilike_escape(self):
+        self.assert_compile(
+            column('x').ilike('a%b_c', escape='\\'),
+            "lower(x) LIKE lower(:x_1) ESCAPE '\\'",
+            checkparams={'x_1': 'a%b_c'}
+        )
+
+    def test_notlike(self):
+        self.assert_compile(
+            column('x').notlike('y'),
+            "x NOT LIKE :x_1",
+            checkparams={'x_1': 'y'}
+        )
+
+    def test_notlike_escape(self):
+        self.assert_compile(
+            column('x').notlike('a%b_c', escape='\\'),
+            "x NOT LIKE :x_1 ESCAPE '\\'",
+            checkparams={'x_1': 'a%b_c'}
+        )
+
+    def test_notilike(self):
+        self.assert_compile(
+            column('x').notilike('y'),
+            "lower(x) NOT LIKE lower(:x_1)",
+            checkparams={'x_1': 'y'}
+        )
+
+    def test_notilike_escape(self):
+        self.assert_compile(
+            column('x').notilike('a%b_c', escape='\\'),
+            "lower(x) NOT LIKE lower(:x_1) ESCAPE '\\'",
+            checkparams={'x_1': 'a%b_c'}
+        )
+
     def test_startswith(self):
         self.assert_compile(
             column('x').startswith('y'),
-            "x LIKE :x_1 || '%%'",
+            "x LIKE :x_1 || '%'",
             checkparams={'x_1': 'y'}
         )
 
     def test_startswith_escape(self):
         self.assert_compile(
-            column('x').startswith('y', escape='\\'),
-            "x LIKE :x_1 || '%%' ESCAPE '\\'",
-            checkparams={'x_1': 'y'}
+            column('x').startswith('a%b_c', escape='\\'),
+            "x LIKE :x_1 || '%' ESCAPE '\\'",
+            checkparams={'x_1': 'a%b_c'}
+        )
+
+    def test_startswith_autoescape(self):
+        self.assert_compile(
+            column('x').startswith('a%b_c/d', autoescape=True),
+            "x LIKE :x_1 || '%' ESCAPE '/'",
+            checkparams={'x_1': 'a/%b/_c//d'}
+        )
+
+    def test_startswith_autoescape_custom_escape(self):
+        self.assert_compile(
+            column('x').startswith('a%b_c/d^e', autoescape=True, escape='^'),
+            "x LIKE :x_1 || '%' ESCAPE '^'",
+            checkparams={'x_1': 'a^%b^_c/d^^e'}
         )
 
     def test_not_startswith(self):
         self.assert_compile(
             ~column('x').startswith('y'),
-            "x NOT LIKE :x_1 || '%%'",
+            "x NOT LIKE :x_1 || '%'",
             checkparams={'x_1': 'y'}
         )
 
     def test_not_startswith_escape(self):
         self.assert_compile(
-            ~column('x').startswith('y', escape='\\'),
-            "x NOT LIKE :x_1 || '%%' ESCAPE '\\'",
-            checkparams={'x_1': 'y'}
+            ~column('x').startswith('a%b_c', escape='\\'),
+            "x NOT LIKE :x_1 || '%' ESCAPE '\\'",
+            checkparams={'x_1': 'a%b_c'}
+        )
+
+    def test_not_startswith_autoescape(self):
+        self.assert_compile(
+            ~column('x').startswith('a%b_c/d', autoescape=True),
+            "x NOT LIKE :x_1 || '%' ESCAPE '/'",
+            checkparams={'x_1': 'a/%b/_c//d'}
         )
 
     def test_startswith_literal(self):
         self.assert_compile(
             column('x').startswith(literal_column('y')),
-            "x LIKE y || '%%'",
+            "x LIKE y || '%'",
             checkparams={}
         )
 
     def test_startswith_text(self):
         self.assert_compile(
             column('x').startswith(text('y')),
-            "x LIKE y || '%%'",
+            "x LIKE y || '%'",
             checkparams={}
         )
 
@@ -2381,42 +2574,79 @@ class ComposedLikeOperatorsTest(fixtures.TestBase, testing.AssertsCompiledSQL):
     def test_endswith(self):
         self.assert_compile(
             column('x').endswith('y'),
-            "x LIKE '%%' || :x_1",
+            "x LIKE '%' || :x_1",
             checkparams={'x_1': 'y'}
         )
 
     def test_endswith_escape(self):
         self.assert_compile(
-            column('x').endswith('y', escape='\\'),
-            "x LIKE '%%' || :x_1 ESCAPE '\\'",
-            checkparams={'x_1': 'y'}
+            column('x').endswith('a%b_c', escape='\\'),
+            "x LIKE '%' || :x_1 ESCAPE '\\'",
+            checkparams={'x_1': 'a%b_c'}
+        )
+
+    def test_endswith_autoescape(self):
+        self.assert_compile(
+            column('x').endswith('a%b_c/d', autoescape=True),
+            "x LIKE '%' || :x_1 ESCAPE '/'",
+            checkparams={'x_1': 'a/%b/_c//d'}
+        )
+
+    def test_endswith_autoescape_custom_escape(self):
+        self.assert_compile(
+            column('x').endswith('a%b_c/d^e', autoescape=True, escape="^"),
+            "x LIKE '%' || :x_1 ESCAPE '^'",
+            checkparams={'x_1': 'a^%b^_c/d^^e'}
+        )
+
+    def test_endswith_autoescape_warning(self):
+        with expect_warnings("The autoescape parameter is now a simple"):
+            self.assert_compile(
+                column('x').endswith('a%b_c/d', autoescape='P'),
+                "x LIKE '%' || :x_1 ESCAPE '/'",
+                checkparams={'x_1': 'a/%b/_c//d'}
+            )
+
+    def test_endswith_autoescape_nosqlexpr(self):
+        assert_raises_message(
+            TypeError,
+            "String value expected when autoescape=True",
+            column('x').endswith,
+            literal_column("'a%b_c/d'"), autoescape=True
         )
 
     def test_not_endswith(self):
         self.assert_compile(
             ~column('x').endswith('y'),
-            "x NOT LIKE '%%' || :x_1",
+            "x NOT LIKE '%' || :x_1",
             checkparams={'x_1': 'y'}
         )
 
     def test_not_endswith_escape(self):
         self.assert_compile(
-            ~column('x').endswith('y', escape='\\'),
-            "x NOT LIKE '%%' || :x_1 ESCAPE '\\'",
-            checkparams={'x_1': 'y'}
+            ~column('x').endswith('a%b_c', escape='\\'),
+            "x NOT LIKE '%' || :x_1 ESCAPE '\\'",
+            checkparams={'x_1': 'a%b_c'}
+        )
+
+    def test_not_endswith_autoescape(self):
+        self.assert_compile(
+            ~column('x').endswith('a%b_c/d', autoescape=True),
+            "x NOT LIKE '%' || :x_1 ESCAPE '/'",
+            checkparams={'x_1': 'a/%b/_c//d'}
         )
 
     def test_endswith_literal(self):
         self.assert_compile(
             column('x').endswith(literal_column('y')),
-            "x LIKE '%%' || y",
+            "x LIKE '%' || y",
             checkparams={}
         )
 
     def test_endswith_text(self):
         self.assert_compile(
             column('x').endswith(text('y')),
-            "x LIKE '%%' || y",
+            "x LIKE '%' || y",
             checkparams={}
         )
 
@@ -2464,6 +2694,38 @@ class CustomOpTest(fixtures.TestBase):
         assert operators.is_comparison(op1)
         assert not operators.is_comparison(op2)
 
+    def test_return_types(self):
+        some_return_type = sqltypes.DECIMAL()
+
+        for typ in [
+            sqltypes.NULLTYPE,
+            Integer(),
+            ARRAY(String),
+            String(50),
+            Boolean(),
+            DateTime(),
+            sqltypes.JSON(),
+            postgresql.ARRAY(Integer),
+            sqltypes.Numeric(5, 2),
+        ]:
+            c = column('x', typ)
+            expr = c.op('$', is_comparison=True)(None)
+            is_(expr.type, sqltypes.BOOLEANTYPE)
+
+            c = column('x', typ)
+            expr = c.bool_op('$')(None)
+            is_(expr.type, sqltypes.BOOLEANTYPE)
+
+            expr = c.op('$')(None)
+            is_(expr.type, typ)
+
+            expr = c.op('$', return_type=some_return_type)(None)
+            is_(expr.type, some_return_type)
+
+            expr = c.op(
+                '$', is_comparison=True, return_type=some_return_type)(None)
+            is_(expr.type, some_return_type)
+
 
 class TupleTypingTest(fixtures.TestBase):
 
@@ -2515,6 +2777,15 @@ class AnyAllTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             checkparams={"param_1": 5}
         )
 
+    def test_any_array_method(self):
+        t = self._fixture()
+
+        self.assert_compile(
+            5 == t.c.arrval.any_(),
+            ":param_1 = ANY (tab1.arrval)",
+            checkparams={"param_1": 5}
+        )
+
     def test_all_array(self):
         t = self._fixture()
 
@@ -2524,6 +2795,15 @@ class AnyAllTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             checkparams={"param_1": 5}
         )
 
+    def test_all_array_method(self):
+        t = self._fixture()
+
+        self.assert_compile(
+            5 == t.c.arrval.all_(),
+            ":param_1 = ALL (tab1.arrval)",
+            checkparams={"param_1": 5}
+        )
+
     def test_any_comparator_array(self):
         t = self._fixture()
 
@@ -2632,6 +2912,16 @@ class AnyAllTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             checkparams={'data_1': 10, 'param_1': 5}
         )
 
+    def test_any_subq_method(self):
+        t = self._fixture()
+
+        self.assert_compile(
+            5 == select([t.c.data]).where(t.c.data < 10).as_scalar().any_(),
+            ":param_1 = ANY (SELECT tab1.data "
+            "FROM tab1 WHERE tab1.data < :data_1)",
+            checkparams={'data_1': 10, 'param_1': 5}
+        )
+
     def test_all_subq(self):
         t = self._fixture()
 
@@ -2642,3 +2932,12 @@ class AnyAllTest(fixtures.TestBase, testing.AssertsCompiledSQL):
             checkparams={'data_1': 10, 'param_1': 5}
         )
 
+    def test_all_subq_method(self):
+        t = self._fixture()
+
+        self.assert_compile(
+            5 == select([t.c.data]).where(t.c.data < 10).as_scalar().all_(),
+            ":param_1 = ALL (SELECT tab1.data "
+            "FROM tab1 WHERE tab1.data < :data_1)",
+            checkparams={'data_1': 10, 'param_1': 5}
+        )
